---
title: "AQM0802A に文字を表示してみる"
description: "『ゲーム電卓パイ』プロジェクト～その２：LCD キャラクタディスプレイモジュール AQM0802A を試してみる"
date: 2022-06-21
thumb: "ogp_aqm0802a.webp"
tags:
    - オコゲ
    - デスクトップアプリ版
---

{% talkicon_r {img: 'mimmy.webp', name: 'ミミィ'} %}{% says_r %}
とにかく試しに動かしてみるわよ！  
考えてるヒマがあったら手を動かすのよ手を！
{% endsays_r %}

{% talkicon_l {img: 'tama.webp', name: 'タマちー'} %}{% says_l %}
１行目に「ocoge」、２行目に「club」と表示してみよう・・・
{% endsays_l %}

{% simplebox 'box_stripe' %}手順説明の下に『オコゲ』のブロックで組み立てたプログラムコードを畳んで置いてあります。クリックすると開きます。  
※一部ブロックの形状が現行バージョンと異なります。{% endsimplebox %}

{% talkicon_r {img: 'mimmy.webp', name: ''} %}{% says_r %}
それとこの先の説明では16進数と2進数が何の説明もなく出てくるわ！  
気になる人は {% link_new {uri: 'https://wa3.i-3-i.info/word1610.html',title: ''} %}16進数{% endlink_new %}, {% link_new {uri: 'https://wa3.i-3-i.info/word1606.html',title: ''} %}2進数{% endlink_new %} あたりを見てネ！
{% endsays_r %}

{% talkicon_l {img: 'tama.webp', name: ''} %}{% think_l %}
【{% link_new {uri: 'https://wa3.i-3-i.info/index.html',title: ''} %}「分かりそう」で「分からない」でも「分かった」気になれるIT用語辞典{% endlink_new %}】  
ユルくて好き・・・
{% endthink_l %}

## I2C アドレス

次のコマンドで確認します。

```bash
$ i2cdetect -y 6
     0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f
00:                         -- -- -- -- -- -- -- -- 
10: -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
20: -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
30: -- -- -- -- -- -- -- -- -- -- -- -- -- -- 3e -- 
40: -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
50: -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
60: -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
70: -- -- -- -- -- -- -- -- 
```

AQM0802A の I2C アドレスは 0x3e と分かりました。

{% blockprogram {
    summary: 'I2C デバイスを開く'
} %}
{% centerimg 'game-dentak_i2copen.webp' %}
GPIO を有効にして I2C デバイスに接続する
{% endcenterimg %}

デバイスに限らず、何かに接続して使用可能な状態にすることをコンピュータ用語では「開く (open)」と表現します。
{% endblockprogram %}

## コマンド書き込み

レジスタ 0x00 にコマンドコード（１バイト）を書き込むことで、AQM0802A にコマンド（命令）を送信することができます。

{% blockprogram {
    summary: 'コマンドを書き込む'
} %}
繰り返し使用するプログラムなので、関数として分離しておきます。

{% centerimg 'game-dentak_func-command.webp' %}
コマンド書き込み関数
{% endcenterimg %}

コマンド書き込み毎に待ち時間が必要です。待ち時間の長さはコマンドの種類によって若干の違いがあるようですが、ここでは無視して１００分の１秒で統一します。

{% talkicon_r {img: 'mimmy.webp', name: ''} %}{% says_r %}
ちゃんと動いているから大丈夫！  
たぶん！
{% endsays_r %}
{% endblockprogram %}

## データ書き込み

レジスタ 0x40 にバイトコードを書き込むことで、AQM0802A 内蔵の RAM (DDRAM/CGRAM) にデータを書き込むことができます。DDRAM にデータを書き込むと画面に文字を表示することができます。CGRAM は自作文字のデータを登録しておくことができます。

{% blockprogram {
    summary: 'データを書き込む'
} %}
こちらも繰り返し使用するので関数にしておきます。

{% centerimg 'game-dentak_func-data.webp' %}
データ書き込み関数
{% endcenterimg %}

{% endblockprogram %}

## 初期化

{% link_new {uri: 'https://akizukidenshi.com/download/ds/xiamen/AQM0802.pdf',title: ''} %}秋月電子のデータシート{% endlink_new %}に従って初期化します。

以下のコマンドコードを、順にレジスタ 0x00 に書き込みます。

1. 0x38 (機能設定：表示行数とか文字の大きさとか)
1. 0x39 (拡張コマンドモード開始)
1. 0x14 (内蔵発振器周波数)
1. 0x70 (コントラスト)
1. 0x56 (3V動作)
1. 0x6c (フォロワ回路のオンオフと増幅率)
1. 0x38 (拡張コマンドモード終了)
1. 0x0c (ディスプレイオン・カーソルオフ)
1. 0x01 (ディスプレイクリア)

{% blockprogram {
    summary: 'AQM0802A を初期化'
} %}
{% centerimg 'game-dentak_init.webp' %}
初期化手順
{% endcenterimg %}
{% endblockprogram %}

## 文字表示

DDRAM にデータを書き込むと画面に文字を表示します。

### DDRAM アドレスを指定

文字を表示するディスプレイ上の位置（２行目の３文字目とか）を指定します。

（データシートコマンドコード表より）
| Instruction       |DB7|DB6|DB5|DB4|DB3|DB2|DB1|DB0|
|-------------------|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|
| Set DDRAM address | 1 |AC6|AC5|AC4|AC3|AC2|AC1|AC0|

データシートではコマンドは8桁（8ビット）の2進数で書かれています。それぞれのビットに役割があり、最上位ビットを立てると DDRAM アドレス指定コマンドとして働きます。残りの７ビットでアドレス値を指定します。DDRAM アドレス値と表示位置の対応は次の通りです。（横の〇文字目を便宜上「列」と表現することにします）

|     |1列目|2列目|3列目|4列目|5列目|6列目|7列目|8列目|
|:---:|:--:|:--:|:--:|:--:|:--:|:--:|:--:|:--:|
|１行目|0x00|0x01|0x02|0x03|0x04|0x05|0x06|0x07|
|２行目|0x40|0x41|0x42|0x43|0x44|0x45|0x46|0x47|

以上から、ディスプレイ上の文字表示位置を指定するには以下のようにします。

* レジスタ 0x00 に、「0x80（最上位ビット） +（上の表のアドレス値）」を書き込む

### 文字表示

DDRAM アドレス指定コマンドを書き込んだのちにデータ書き込みを行うことで、指定した位置に文字を表示することができます。文字コードは以下の通りです。

{% floatbox {
    img: 'game-dentak_character-patterns.webp',
    title: 'キャラクターパターン表',
    width: '400px',
    colortheme: 'silver'
} %}
<i class="fa-solid fa-magnifying-glass"></i> クリックで拡大
{% endfloatbox %}

コード `0010 0000` からコード `0111 1101` まで、すなわちキーボードで入力できる半角（1バイト）の英数字・記号の大部分は UTF-8 (ascii) のコードと一致していますので、表示したい文字列がこの範囲内であれば、そのままデータとして（レジスタ 0x40 へ）書き込めば良いでしょう。

{% blockprogram {
    summary: 'AQM0802A に文字を表示する'
} %}
行と列を指定して文字列表示を行う関数を作成しましょう。

{% floatbox {
    img: 'game-dentak_func-dispchars.webp',
    title: '文字列表示関数',
    width: '800px',
    colortheme: 'silver'
} %}
<i class="fa-solid fa-magnifying-glass"></i> 文字列表示関数
{% endfloatbox %}

「行」と「列」は、内部的には０（ゼロ）から始まる数え方をするので、それぞれ１を引きます。

大文字・小文字・記号などがちゃんと表示されるかどうか、表示位置指定の確認も含めて、１行目に「{ocoge}」、２行目は右に３文字ずらして４文字目から「CLUB!」と５秒間表示することにしましょう。

{% centerimg 'game-dentak_call-dispchars.webp' %}
{% endcenterimg %}

最後に、ディスプレイをオフにしてデバイスとの接続を切るなど後片付けをします。ディスプレイの表示をオフにするにはコマンド `0x08` を書き込みます。

{% centerimg 'game-dentak_okatazuke.webp' %}
{% endcenterimg %}

{% endblockprogram %}

## まとめ

今回作成したプログラムはこのようになりました。

{% floatbox {
    img: 'game-dentak_display-characters.webp',
    title: 'AQM0802A に文字を表示してみるプログラム',
    width: '400px',
    colortheme: 'silver'
} %}
<i class="fa-solid fa-magnifying-glass"></i> AQM0802A に文字を表示してみるプログラム
{% endfloatbox %}

{% floatbox {
    img: 'game-dentak_display-characters-result.webp',
    title: 'AQM0802A に文字を表示してみた結果',
    width: '200px',
    colortheme: 'silver'
} %}
<i class="fa-solid fa-magnifying-glass"></i> 実行結果
{% endfloatbox %}

バックライトは点灯させていません。期待通りの動作をしているようです。メデタシ。  
次回はゲームのルール確認と自作文字（UFO と残機表示）の作成をしましょう。